/**
 *
 * Copyright 2013 Vibes.IO
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var crypto = require('crypto'),
    Url = require('url'),
    util = require('util'),
    Canonical = require('./canonical_json');

module.exports = (function() {

    function Signer(options) {
        if(!(this instanceof Signer))
            return new Signer(options);


        this.key = options.key;
        this.token = options.token;
        this.algo = options.algo || 'sha1';
    }

    /**
     * @method sign
     * @description Create a valid signature
     * @param path
     * @param params
     * @param headers
     * @param body
     */
    Signer.prototype.sign = function(path, params, headers, body)  {
        var hmac = crypto.createHmac(this.algo, this.key);

        var req = {
            path:path,
            params:params,
            headers:headers,
            body:body,
            token: this.token
        };

        // Sign the request object. We use a canonical form to ensure the object is always serialized the same way. That's no guaranteed with JSON.stringify
        hmac.update(Canonical(req));
        return hmac.digest('hex');
    };

    Signer.prototype.signUrl = function(url, params, headers, body) {
        var parsedUrl = Url.parse(url);
        return this.sign(parsedUrl.pathname, params, headers, body);
    };

    Signer.prototype.buildAuthorizationHeader = function(sign) {
        return util.format('Vibes-Signature token="%s", algo="%s", signature="%s"', this.token, this.algo, sign);
    };

    return Signer;
})();
